@@ -1310,12 +1310,15 @@ static gps_mask_t sirf_msg_swversion(struct gps_device_t *session,
     return DEVICEID_SET;
 }
 
-/* subframe data MID 8 */
+/* subframe data MID 8
+ * Use Message ID 14 and 15 instead?
+ * Use Message ID 56, Sub ID 5, instead? */
 static gps_mask_t sirf_msg_navdata(struct gps_device_t *session,
                                    unsigned char *buf, size_t len)
 {
     unsigned int i, chan, svid;
     uint32_t words[10];
+    const unsigned int numwords = 10;
 
     if (len != 43)
         return 0;
@@ -1323,7 +1326,8 @@ static gps_mask_t sirf_msg_navdata(struct gps_device_t *session,
     chan = (unsigned int)getub(buf, 1);
     svid = (unsigned int)getub(buf, 2);
 
-    for (i = 0; i < 10; i++) {
+    for (i = 0; i < numwords; i++) {
+        // packing undocumented...
         words[i] = (uint32_t)getbeu32(buf, 4 * i + 3);
     }
 
@@ -1335,6 +1339,7 @@ static gps_mask_t sirf_msg_navdata(struct gps_device_t *session,
         /* some USB are also too slow, no way to tell which ones */
         if (session->context->readonly ||
             session->context->passive) {
+            // FIXME: duplicated in gpsd_internet_subframe_raw()
             GPSD_LOG(LOG_WARN, &session->context->errout,
                      "WARNING: SiRF: link too slow.\n");
         } else {
@@ -1344,7 +1349,8 @@ static gps_mask_t sirf_msg_navdata(struct gps_device_t *session,
         }
     }
 
-    return gpsd_interpret_subframe_raw(session, svid, words);
+    return gpsd_interpret_subframe_raw(session, GNSSID_GPS, svid, words,
+                                       numwords);
 }
 
 /* max channels allowed in old MID 4 SiRF format */
@@ -1472,7 +1478,7 @@ static double sirf_time_offset(struct gps_device_t *session)
 
     /* the Navigation Solution message */
     else if (session->driver.sirf.lastid == (unsigned char)2) {
-        if (session->sourcetype == source_usb) {
+        if (SOURCE_USB == session->sourcetype) {
             retval = 0.640;     /* USB, expect +/- 50mS jitter */
         } else {
             switch (session->gpsdata.dev.baudrate) {
